#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library
import logging
logger = logging.getLogger('PyAMS (utils)')

# import interfaces
from zope.schema.interfaces import IVocabularyFactory

# import packages
from zope.interface import directlyProvides
from zope.schema.vocabulary import getVocabularyRegistry


class vocabulary_config:
    """Class decorator to define a vocabulary

    :param str name: name of the registered vocabulary

    This is, for example, how a vocabulary of registered ZEO connections utilities is created:

    .. code-block:: python

        from pyams_utils.interfaces.zeo import IZEOConnection

        from pyams_utils.registry import get_utilities_for
        from pyams_utils.vocabulary import vocabulary_config
        from zope.schema.vocabulary import SimpleTerm, SimpleVocabulary

        @vocabulary_config(name='PyAMS ZEO connections')
        class ZEOConnectionVocabulary(SimpleVocabulary):
            '''ZEO connections vocabulary'''

            def __init__(self, context=None):
                terms = [SimpleTerm(name, title=util.name) for name, util in get_utilities_for(IZEOConnection)]
                super(ZEOConnectionVocabulary, self).__init__(terms)

    You can then use such a vocabulary in any schema field:

    .. code-block:: python

        from zope.interface import Interface
        from zope.schema import Choice

        class MySchema(Interface):
            '''Custom schema interface'''

            zeo_connection_name = Choice(title='ZEO connection name',
                                         description='Please select a registered ZEO connection',
                                         vocabulary='PyAMS ZEO connections',
                                         required=False)
    """

    def __init__(self, name):
        self.name = name

    def __call__(self, klass):
        logger.debug('Registering class {0} as vocabulary with name "{1}"'.format(str(klass), self.name))
        directlyProvides(klass, IVocabularyFactory)
        getVocabularyRegistry().register(self.name, klass)
        return klass
